// 
// Copyright (c) 2004-2011 Jaroslaw Kowalski <jaak@jkowalski.net>
// 
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions 
// are met:
// 
// * Redistributions of source code must retain the above copyright notice, 
//   this list of conditions and the following disclaimer. 
// 
// * Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution. 
// 
// * Neither the name of Jaroslaw Kowalski nor the names of its 
//   contributors may be used to endorse or promote products derived from this
//   software without specific prior written permission. 
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
// THE POSSIBILITY OF SUCH DAMAGE.
// 

namespace NLog.UnitTests.LayoutRenderers.Wrappers
{
    using NLog;
    using NUnit.Framework;

#if !NUNIT
    using SetUp = Microsoft.VisualStudio.TestTools.UnitTesting.TestInitializeAttribute;
    using TestFixture = Microsoft.VisualStudio.TestTools.UnitTesting.TestClassAttribute;
    using Test = Microsoft.VisualStudio.TestTools.UnitTesting.TestMethodAttribute;
    using TearDown =  Microsoft.VisualStudio.TestTools.UnitTesting.TestCleanupAttribute;
#endif
    using NLog.Layouts;

    [TestFixture]
    public class FileSystemNormalizeTests : NLogTestBase
    {
        [Test]
        public void FSNormalizeTest1()
        {
            MappedDiagnosticsContext.Clear();
            MappedDiagnosticsContext.Set("foo", "abc.log");
            SimpleLayout l = "${filesystem-normalize:${mdc:foo}}";
            
            Assert.AreEqual("abc.log", l.Render(LogEventInfo.CreateNullEvent()));

            MappedDiagnosticsContext.Set("foo", "");
            Assert.AreEqual("", l.Render(LogEventInfo.CreateNullEvent()));

            MappedDiagnosticsContext.Set("foo", "a/b/c");
            Assert.AreEqual("a_b_c", l.Render(LogEventInfo.CreateNullEvent()));

            // all characters outside of alpha/digits/space/_/./- are replaced with _
            MappedDiagnosticsContext.Set("foo", ":\\/$@#$%^");
            Assert.AreEqual("_________", l.Render(LogEventInfo.CreateNullEvent()));
        }

        [Test]
        public void FSNormalizeTest2()
        {
            MappedDiagnosticsContext.Clear();
            MappedDiagnosticsContext.Set("foo", "abc.log");
            SimpleLayout l = "${mdc:foo:fsnormalize=true}";

            Assert.AreEqual("abc.log", l.Render(LogEventInfo.CreateNullEvent()));

            MappedDiagnosticsContext.Set("foo", "");
            Assert.AreEqual("", l.Render(LogEventInfo.CreateNullEvent()));

            MappedDiagnosticsContext.Set("foo", "a/b/c");
            Assert.AreEqual("a_b_c", l.Render(LogEventInfo.CreateNullEvent()));

            // all characters outside of alpha/digits/space/_/./- are replaced with _
            MappedDiagnosticsContext.Set("foo", ":\\/$@#$%^");
            Assert.AreEqual("_________", l.Render(LogEventInfo.CreateNullEvent()));
        }
    }
}